CREATE OR REPLACE FUNCTION process_node(node jsonb)
RETURNS text AS $$
DECLARE
    node_type text := node->>'Node Type';
    relname text;
    filter_expr text;
    join_cond text;
    colname text;
    result text := '';
    child jsonb;
    child_result text;
    existing_indexes text[];
    sort_key text;
    group_key text;
    match text[];
    table_size bigint;
    is_large_table boolean := false;
    join_type text;
    actual_rows double precision;
    planned_rows double precision;
    startup_cost float;
    total_cost float;
    has_expensive_node boolean := false;
    colnames text[];
    join_columns text[];
    sort_columns text[];
    group_columns text[];
BEGIN
    -- Check if node is expensive (cost > 10000 or row discrepancy > 10x)
    startup_cost := (node->>'Startup Cost')::float;
    total_cost := (node->>'Total Cost')::float;
    planned_rows := (node->>'Plan Rows')::float;
    actual_rows := (node->>'Actual Rows')::float;
    
    IF total_cost > 10000 OR (actual_rows > 0 AND planned_rows > 0 AND 
       (actual_rows/planned_rows > 10 OR planned_rows/actual_rows > 10)) THEN
        has_expensive_node := true;
        result := result || format(
            E'POTENTIAL PERFORMANCE ISSUE: Node type %s has high cost (%.2f) or row estimate discrepancy (planned: %.2f, actual: %.2f)\n',
            node_type, total_cost, planned_rows, actual_rows
        );
    END IF;

    -- Seq Scan analysis with table size consideration
    IF node_type = 'Seq Scan' THEN
        relname := node->>'Relation Name';
        filter_expr := node->>'Filter';
        
        -- Get table size
        EXECUTE format('SELECT pg_total_relation_size(%L)',
                      relname) INTO table_size;
        is_large_table := table_size > 100 * 1024 * 1024; -- 100MB threshold
        
        -- Filter analysis
        IF filter_expr IS NOT NULL THEN
            -- Extract column names from filter expression
            SELECT array_agg(DISTINCT word) INTO colnames
            FROM regexp_split_to_table(
                regexp_replace(filter_expr, '\(|\)|\"|\''', '', 'g'),
                '\W+'
            ) AS word
            WHERE word ~ '^[a-zA-Z_][a-zA-Z0-9_]*$'
            AND NOT word ~ '^AND$|^OR$|^NOT$|^IN$|^IS$|^NULL$|^LIKE$'
            AND EXISTS (
                SELECT 1 FROM information_schema.columns 
                WHERE table_name = relname AND column_name = word
            );
            
            -- Check for function-based filters
            IF filter_expr ~ '\(.*\)' THEN
                result := result || format(
                    E'FUNCTIONAL EXPRESSION: Filter "%s" contains function calls.\nConsider creating a functional index if this filter is used frequently.\nExample: CREATE INDEX idx_%s_expr ON %I USING btree((%s));\n\n',
                    filter_expr, relname, relname, filter_expr
                );
            END IF;
            
            -- Single column index recommendations
            FOREACH colname IN ARRAY colnames LOOP
                -- Check if index exists
                IF NOT EXISTS (
                    SELECT 1 FROM pg_indexes
                    WHERE tablename = relname
                    AND indexdef ~* format('\(.*%s.*\)', colname)
                ) THEN
                    result := result || format(
                        E'FILTER CONDITION: Table %I has filter on column %I (%s)%s\nRecommendation: CREATE INDEX idx_%s_%s ON %I(%I);\n%s\n',
                        relname, colname, filter_expr,
                        CASE WHEN is_large_table THEN ' (large table)' ELSE '' END,
                        relname, colname, relname, colname,
                        CASE WHEN is_large_table THEN 
                        'Note: Consider concurrent index creation: CREATE INDEX CONCURRENTLY idx_' || relname || '_' || colname || ' ON ' || relname || '(' || colname || ');'
                        ELSE '' END
                    );
                END IF;
            END LOOP;
            
            -- Multi-column index recommendation (for AND conditions)
            IF array_length(colnames, 1) > 1 AND filter_expr ~ 'AND' THEN
                result := result || format(
                    E'MULTI-COLUMN FILTER: Consider composite index for columns %s\nRecommendation: CREATE INDEX idx_%s_comp ON %I(%s);\nColumn order suggestion: Place high-selectivity columns first\n\n',
                    array_to_string(colnames, ','), relname, relname, array_to_string(colnames, ',')
                );
            END IF;
        END IF;
        
        -- If large table with no filter
        IF is_large_table AND filter_expr IS NULL THEN
            result := result || format(
                E'FULL TABLE SCAN: Large table %I (%s MB) scanned without filter\nRecommendation: \n- Add WHERE conditions to limit scanned rows\n- Consider partitioning if this is a frequent operation\n\n',
                relname, (table_size/1024/1024)::int
            );
        END IF;
    END IF;
    
    -- Join analysis
    IF node_type IN ('Hash Join', 'Merge Join', 'Nested Loop') THEN
        join_type := node_type;
        join_cond := node->>'Join Condition';
        IF join_cond IS NULL THEN
            join_cond := node->>'Hash Cond'; -- For Hash Join
        END IF;
        
        IF join_cond IS NOT NULL THEN
            -- Extract join columns
            SELECT array_agg(DISTINCT trim(both from col)) INTO join_columns
            FROM (
                SELECT split_part(join_cond, '=', 1) AS col
                UNION SELECT split_part(join_cond, '=', 2)
            ) AS cols
            WHERE col ~ '\.';
            
            -- Analyze each join column
            FOREACH colname IN ARRAY join_columns LOOP
                relname := split_part(colname, '.', 1);
                colname := split_part(colname, '.', 2);
                
                -- Check for index on join column
                IF NOT EXISTS (
                    SELECT 1 FROM pg_indexes
                    WHERE tablename = relname
                    AND indexdef ~* format('\(.*%s.*\)', colname)
                ) THEN
                    -- Get table size for join recommendations
                    EXECUTE format('SELECT pg_total_relation_size(%L)',
                                  relname) INTO table_size;
                    is_large_table := table_size > 100 * 1024 * 1024;
                    
                    result := result || format(
                        E'JOIN CONDITION: %s on %I.%I%s\nRecommendation: CREATE INDEX idx_%s_%s_join ON %I(%I);\n%s\n',
                        join_type, relname, colname,
                        CASE WHEN is_large_table THEN ' (large table)' ELSE '' END,
                        relname, colname, relname, colname,
                        CASE WHEN join_type = 'Hash Join' AND is_large_table THEN
                        'Note: For large tables with hash joins, ensure work_mem is sufficiently large'
                        WHEN join_type = 'Nested Loop' AND is_large_table THEN
                        'Warning: Nested loop joins on large tables can be inefficient. Consider query rewrite.'
                        ELSE '' END
                    );
                END IF;
            END LOOP;
        END IF;
    END IF;
    
    -- Sort analysis
    IF node_type = 'Sort' THEN
        sort_key := node->>'Sort Key';
        IF sort_key IS NOT NULL THEN
            -- Extract sort columns
            SELECT array_agg(DISTINCT trim(both from col)) INTO sort_columns
            FROM regexp_split_to_table(sort_key, ',') AS col;
            
            -- Analyze each sort column
            FOREACH colname IN ARRAY sort_columns LOOP
                IF colname ~ '\.' THEN
                    relname := split_part(colname, '.', 1);
                    colname := split_part(colname, '.', 2);
                    
                    -- Check for sort index
                    IF NOT EXISTS (
                        SELECT 1 FROM pg_indexes
                        WHERE tablename = relname
                        AND indexdef ~* format('\(.*%s.*\)', colname)
                        AND indexdef ~* 'USING btree'
                    ) THEN
                        -- Check if this is for ORDER BY or GROUP BY
                        IF EXISTS (
                            SELECT 1 FROM jsonb_array_elements(node->'Plans') AS child
                            WHERE child->>'Node Type' = 'GroupAggregate'
                        ) THEN
                            result := result || format(
                                E'GROUP BY SORTING: Table %I sorted on %I for grouping\nRecommendation: Create index matching GROUP BY clause order\nCREATE INDEX idx_%s_%s_group ON %I(%I);\n\n',
                                relname, colname, relname, colname, relname, colname
                            );
                        ELSE
                            result := result || format(
                                E'SORT OPERATION: Table %I sorted on %I\nRecommendation: CREATE INDEX idx_%s_%s_sort ON %I USING btree(%I);\nConsider adding INCLUDE columns if query only needs a few columns\n\n',
                                relname, colname, relname, colname, relname, colname
                            );
                        END IF;
                    END IF;
                END IF;
            END LOOP;
        END IF;
    END IF;
    
    -- GroupAggregate analysis
    IF node_type = 'GroupAggregate' THEN
        group_key := node->>'Group Key';
        IF group_key IS NOT NULL THEN
            -- Extract group columns
            SELECT array_agg(DISTINCT trim(both from col)) INTO group_columns
            FROM regexp_split_to_table(regexp_replace(group_key, '[\[\]]', '', 'g'), ',') AS col;
            
            -- Find the source relation
            SELECT r.relname INTO relname
            FROM jsonb_array_elements(node->'Plans') AS child
            JOIN pg_class r ON r.relname = child->>'Relation Name'
            LIMIT 1;
            
            IF relname IS NOT NULL THEN
                -- Check for existing group index
                IF NOT EXISTS (
                    SELECT 1 FROM pg_indexes
                    WHERE tablename = relname
                    AND indexdef ~* format('\(.*%s.*\)', array_to_string(group_columns, '.*'))
                ) THEN
                    result := result || format(
                        E'GROUPING OPERATION: Grouping on columns %s\nRecommendation: Create dedicated index for grouping:\nCREATE INDEX idx_%s_group ON %I(%s);\n\n',
                        array_to_string(group_columns, ','), relname, relname, array_to_string(group_columns, ',')
                    );
                END IF;
            END IF;
        END IF;
    END IF;
    
    -- Subquery analysis
    IF node_type = 'Subquery Scan' THEN
        result := result || E'SUBQUERY DETECTED: Consider rewriting as:\n';
        result := result || E'1. A JOIN if correlated\n';
        result := result || E'2. A LATERAL join if needing row-by-row processing\n';
        result := result || E'3. A CTE (WITH clause) if complex logic needs isolation\n';
        result := result || E'4. A materialized view if reused frequently\n\n';
    END IF;
    
    -- CTE scan analysis
    IF node_type = 'CTE Scan' THEN
        result := result || format(
            E'CTE SCAN DETECTED: CTE "%s"\nRecommendations:\n',
            node->>'CTE Name'
        );
        
        -- Check if CTE is referenced multiple times
        IF (node->'CTE Name') IS NOT NULL THEN
            result := result || E'- If CTE is referenced multiple times, consider MATERIALIZED\n';
        END IF;
        
        -- Check for filters on CTE
        IF node->>'Filter' IS NOT NULL THEN
            result := result || E'- Push filter conditions into the CTE definition if possible\n';
        END IF;
        
        result := result || E'\n';
    END IF;
    
    -- Recursively process child plans
    IF node ? 'Plans' THEN
        FOR child IN SELECT * FROM jsonb_array_elements(node->'Plans')
        LOOP
            child_result := process_node(child);
            IF child_result IS NOT NULL THEN
                result := result || child_result;
            END IF;
        END LOOP;
    END IF;
    
    RETURN result;
END;
$$ LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION pg_sqltune(query text)
RETURNS text AS $$
DECLARE
    plan jsonb;
    suggestion text := '';
    settings text := '';
    query_hash bigint;
    query_id text;
BEGIN
    -- Create a unique identifier for this query
    query_hash := hashtext(query);
    query_id := md5(query);
    
    -- First try EXPLAIN ANALYZE, fall back to EXPLAIN if it fails
    BEGIN
        EXECUTE format('EXPLAIN (ANALYZE, VERBOSE, FORMAT JSON) %s', query) INTO plan;
    EXCEPTION WHEN OTHERS THEN
        BEGIN
            EXECUTE format('EXPLAIN (VERBOSE, FORMAT JSON) %s', query) INTO plan;
        EXCEPTION WHEN OTHERS THEN
            RETURN format('Error explaining query: %s', SQLERRM);
        END;
    END;
    
    -- Process the plan
    IF plan IS NULL OR jsonb_array_length(plan) = 0 THEN
        RETURN 'Could not generate execution plan for the query';
    END IF;
    
    plan := plan->0->'Plan';
    suggestion := process_node(plan);
    
    -- Add general recommendations
    suggestion := suggestion || E'\nGENERAL RECOMMENDATIONS:\n';
    suggestion := suggestion || E'- Run VACUUM ANALYZE on tables to update statistics\n';
    suggestion := suggestion || E'- Consider increasing work_mem for complex sorts/hashes\n';
    suggestion := suggestion || E'- Review shared_buffers and maintenance_work_mem settings\n';
    suggestion := suggestion || E'- For large tables, consider partitioning strategies\n';
    suggestion := suggestion || E'- Use prepared statements for frequently run queries\n';
    suggestion := suggestion || E'- Check for unused indexes that could be dropped\n';
    
    -- Add current configuration settings that might affect performance
    settings := settings || E'\nCURRENT SETTINGS:\n';
    settings := settings || format('work_mem = %s\n', current_setting('work_mem'));
    settings := settings || format('shared_buffers = %s\n', current_setting('shared_buffers'));
    settings := settings || format('maintenance_work_mem = %s\n', current_setting('maintenance_work_mem'));
    settings := settings || format('random_page_cost = %s\n', current_setting('random_page_cost'));
    settings := settings || format('effective_cache_size = %s\n', current_setting('effective_cache_size'));
    
    -- Add query identification for tracking
    suggestion := suggestion || format(
        E'\nQUERY ID: %s\nHASH: %s\n\n%s',
        query_id, query_hash, settings
    );
    
    IF suggestion = '' THEN
        RETURN 'No optimization suggestions found. Query appears well-tuned.';
    END IF;
    
    RETURN suggestion;
END;
$$ LANGUAGE plpgsql;